/******************************************************************************/
/*!
\file   EdgeDetection.fx
\author Dan Zeinner
\date   4/10/2010
\brief

	This shader implements the edge detection seen in our game.

All content  2010 DigiPen (USA) Corporation, all rights reserved.
*/
/******************************************************************************/

texture texture0;
float4x4 worldViewProj : WorldViewProjection;
float4x4 world : World;
const float ridgeAngle=90.0f;
const float valleyAngle=90.0f;
const float4 edgeColor=float4(1,1.0, 1.0,1);
const float4 hiddenEdgeColor=float4(1,0,0,1);

uniform extern float3 worldScale, worldRotate;

//lighting
uniform extern float4 gDiffuseMtrl;
uniform extern float4 gDiffuseLight;
uniform extern float4 gAmbientMtrl;
uniform extern float4 gAmbientLight;
uniform extern float4 gSpecularMtrl;
uniform extern float4 gSpecularLight;
uniform extern float  gSpecPower;
uniform extern float3 gLightDirW;
uniform extern float3 gLightPosW;

///Edge detection variables
uniform extern float3 gEyePosW;
uniform extern float2 viewWidthHeight;
uniform extern float edgeThickness;
uniform extern texture edgeTexture;

sampler EdgeSampler
{
    Texture   = edgeTexture;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = WRAP;
    AddressV = CLAMP;        
};

struct VS_INPUT
{
	float3 v0	: POSITION0;
	float3 v1	: POSITION1;
	float3 v2	: POSITION2;
	float3 v3	: POSITION3;
	float3 n0 : NORMAL0;
	float3 n1 : NORMAL1;
	float1 r : TEXCOORD0;		
	float1 i : TEXCOORD1;	
};

struct VS_OUTPUT
{
    float4 pos		 : POSITION;	
    float3 normal	 : TEXCOORD;
	float4 color	 : COLOR;
	float2 tex		 : TEXCOORD1;
};

bool IsContour(float3 nA, float3 nB, float3 v0)
{
	//Check if the vertex is on a contour edge
    float nAEye = dot(nA, (gEyePosW-v0));
    float nBEye = dot(nB, (gEyePosW-v0));
    bool nAEyeB = (nAEye<0);
    bool nBEyeB = (nBEye<0);    
    if(!(nAEyeB && nBEyeB) && (nAEyeB || nBEyeB))
    {
		return true;
    }
    else
    {
		return false;
    }
}

bool IsRidgeValley(float3 nA, float3 nB, float3 v2, float3 v3)
{
	float aDotb = dot(nA, nB);
	float3 v3Diffv2 = v3-v2;
	float vDota = dot(v3Diffv2, nA);
	
	//Check for ridge edge
	if((aDotb < - cos(ridgeAngle)) && (vDota<=0))
	{
		return true;
	}
	//Check for valley edge
	else if((aDotb < - cos(valleyAngle)) && (vDota>0))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool IsBoundary(float3 v0, float3 v3)
{
	return v0==v3;
}

bool IsBackFace(VS_INPUT IN)
{
	float3 nA, nB;
    
    nA = normalize(cross((IN.v1 - IN.v0),(IN.v2-IN.v0)));  
    nB = normalize(cross((IN.v3 - IN.v0),(IN.v1-IN.v0))); 
    
    float3 viewDir1 = normalize(gEyePosW - IN.v0);
    float3 viewDir2 = normalize(gEyePosW - IN.v1);
    
    bool d1 = dot(viewDir1, nA)<0;
    bool d2 = dot(viewDir2, nB)<0;
    
    return d1 && d2;
}

float4 TransformThickVertex(float4 s0, float4 s1, float2 p, float i)
{
	float4 outPos;
	float2 xyPos;
	if(i==0.0)
	{		
		xyPos=((s0.xy-p)/viewWidthHeight)*s0.w;	
		outPos=float4(xyPos.x, xyPos.y, s0.zw);								
	}
	else if(i==1.0)
	{
		xyPos=((s0.xy+p)/viewWidthHeight)*s0.w;	
		outPos=float4(xyPos.x, xyPos.y, s0.zw);			
	}
	else if(i==2.0)
	{
		xyPos=((s1.xy+p)/viewWidthHeight)*s1.w;
		outPos=float4(xyPos.x, xyPos.y, s1.zw);				
	}
	else if(i==3.0)
	{
		xyPos=((s1.xy-p)/viewWidthHeight)*s1.w;	
		outPos=float4(xyPos.x, xyPos.y, s1.zw);				
	}	
	return outPos;	
}

float4 TransformHalfQuadVertex(float4 s0, float4 s1, float2 p, float i)
{
	float4 outPos;
	float2 xyPos;
	if(i==0.0)
	{		
		xyPos=((s0.xy)/viewWidthHeight)*s0.w;	
		outPos=float4(xyPos.x, xyPos.y, s0.zw);						
	}
	else if(i==1.0)
	{
		xyPos=((s0.xy+p)/viewWidthHeight)*s0.w;	
		outPos=float4(xyPos.x, xyPos.y, s0.zw);
	}
	else if(i==2.0)
	{
		xyPos=((s1.xy+p)/viewWidthHeight)*s1.w;
		outPos=float4(xyPos.x, xyPos.y, s1.zw);
	}
	else if(i==3.0)
	{
		xyPos=((s1.xy)/viewWidthHeight)*s1.w;	
		outPos=float4(xyPos.x, xyPos.y, s1.zw);
	}
	return outPos;	
}

float4 TransformCapVertex(float4 s, float4 m, float2 p, float i)
{
	float4 outPos;
	float2 xyPos;
	if(i==0.0)
	{		
		xyPos=((s.xy)/viewWidthHeight)*s.w;	
		outPos=float4(xyPos.x, xyPos.y, s.zw);		
	}
	else if(i==1.0)
	{
		xyPos=((s.xy+p)/viewWidthHeight)*s.w;	
		outPos=float4(xyPos.x, xyPos.y, s.zw);
	}
	else if(i==2.0)
	{
		float2 cornerVert = s.xy+m.xy;
		xyPos=(cornerVert/viewWidthHeight)*s.w;
		outPos=float4(xyPos.x, xyPos.y, s.zw);
	}		
	
	return outPos;
}

bool IsEdge(in VS_INPUT IN)
{
	 //Calculate nA and nB
    //nA is the normal for the face with v0, v1, v2 and nB is the normal for the face with v0,v1,v3
    float3 nA, nB;
    
    nA = normalize(cross((IN.v1 - IN.v0),(IN.v2-IN.v0)));  
    nB = normalize(cross((IN.v3 - IN.v0),(IN.v1-IN.v0)));  
    
    return  IsContour(nA, nB, IN.v0)||
			IsRidgeValley(nA, nB, IN.v2, IN.v3);//||
			//IsBoundary(IN.v0, IN.v3);   
}

float2 CalculateTextureCoordinates(int i, float2 s0, float2 s1, float2 c, int dotSign)
{
	float2 texCoord = float2(0,0);
	if(i==0 || i==3)
	{
		texCoord.y=1;
	}
	else
	{
		texCoord.y=0;
	}
	if(i==0 || i==1)
	{
		texCoord.x=0;
	}
	else
	{
		texCoord.x=1;
	}
	
	return texCoord;
}

VS_OUTPUT ThinEdgeVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;	 
    
    float3 Pos;    
    float3 Normal; 
    //Check if the vertex is on a visible edge
    if(IsEdge(IN))
    {
		if(IN.i==0.0)
		{
			Pos=IN.v0;
			Normal=IN.n0;
			OUT.color=float4(IN.n0,1);		
		}
		else if(IN.i==1.0)
		{
			Pos=IN.v1;
			Normal=IN.n1;
			OUT.color=float4(IN.n1,1);
		}		
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		Pos=float3(0,0,-1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,1,0,1);
    }   
	
	OUT.pos = mul( float4(Pos, 1.0f) , worldViewProj);
	OUT.normal = Normal;		
	OUT.tex = float2(1,1);

	return OUT;
}

VS_OUTPUT ThickEdgeVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;	 
    
    float4 Pos;    
    float3 Normal; 
    
    //Calculate s0 and s1 the screen space projections of the xy components of v0 and v1
    float4 v0 = float4(IN.v0,1.0);
    float4 v1 = float4(IN.v1,1.0);   
    float4 s0 = mul(v0, worldViewProj);
    float4 s1 = mul(v1, worldViewProj);
    
    s0.xy=(s0.xy/s0.w)*viewWidthHeight;
    s1.xy=(s1.xy/s1.w)*viewWidthHeight;     
    
    //Calculate p the screen space perpendicular to the edge
    float2 p = normalize(float2(s0.y-s1.y, s1.x-s0.x))*edgeThickness;       
       
    //Check if the vertex is on a visible edge
    if(IsEdge(IN))// && !IsBackFace(IN))
    {
		OUT.pos=TransformThickVertex(s0, s1, p, IN.i);		
		
		OUT.color=edgeColor;		
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		OUT.pos=float4(0,0,-1,1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,0,0,1);
    }   	
	OUT.normal = Normal;			
	OUT.tex = float2(1,1);	

	return OUT;
}

VS_OUTPUT HalfQuadEdgeVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;	 
    
    float4 Pos;    
    float3 Normal; 
    
    //Calculate s0 and s1 the screen space projections of the xy components of v0 and v1
    float4 v0 = float4(IN.v0,1.0);
    float4 v1 = float4(IN.v1,1.0);   
    float4 n0 = float4(IN.n0, 0.0);
    n0=normalize(n0);
    
    
    float4 s0 = mul(v0, worldViewProj);
    float4 s1 = mul(v1, worldViewProj);
    float4 m0 = mul((v0+n0), worldViewProj);
    
    float4 origin = float4(0,0,0,1);
    
    s0.xy=(s0.xy/s0.w)*viewWidthHeight;
    s1.xy=(s1.xy/s1.w)*viewWidthHeight;
    m0.xy=(m0.xy/m0.w)*viewWidthHeight - s0.xy;         
    origin.xy=(origin.xy/origin.w)*viewWidthHeight;
    
    //Calculate p the screen space perpendicular to the edge
    float2 p = normalize(float2(s0.y-s1.y, s1.x-s0.x))*edgeThickness; 
    
    int dotSign = sign(dot(m0.xy,p));
    
    p*=dotSign;  
    
    if(dotSign<0)
    {		
		if(IN.i==1.0)
		{
			IN.i=3.0;			
		}
		else if(IN.i==3.0)
		{
			IN.i=1.0;
		}
    }    
    
    float2 xyPos;
    //Check if the vertex is on a visible edge
    if(IsEdge(IN)) //&& !IsBackFace(IN))
    {		
		OUT.pos=TransformHalfQuadVertex(s0,s1,p,IN.i);
		OUT.color=edgeColor;
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		OUT.pos=float4(0,0,-1,1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,0,0,1);
    }   	
	OUT.normal = Normal;	
	OUT.tex = CalculateTextureCoordinates(IN.i, s0.xy, s1.xy, origin.xy, dotSign);		

	return OUT;
}

VS_OUTPUT EdgeDetectionVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;	 
    
    float4 Pos;    
    float3 Normal; 
    
    //Calculate s0 and s1 the screen space projections of the xy components of v0 and v1
    float4 v0 = float4(IN.v0,1.0);
    float4 v1 = float4(IN.v1,1.0);   
    float4 n0 = float4(IN.n0, 0.0);
    n0=normalize(n0);   
    
    float3 nA, nB;
    
    nA = normalize(cross((IN.v1 - IN.v0),(IN.v2-IN.v0)));  
    nB = normalize(cross((IN.v3 - IN.v0),(IN.v1-IN.v0)));  
    
    float4 s0 = mul(v0, worldViewProj);
    float4 s1 = mul(v1, worldViewProj);
    float4 m0 = mul((v0+n0), worldViewProj);
    
    s0.xy=(s0.xy/s0.w)*viewWidthHeight;
    s1.xy=(s1.xy/s1.w)*viewWidthHeight;
    m0.xy=(m0.xy/m0.w)*viewWidthHeight - s0.xy;         
    
    //Calculate p the screen space perpendicular to the edge
    float2 p = normalize(float2(s0.y-s1.y, s1.x-s0.x))*edgeThickness;   
    
    int dotSign = sign(dot(m0.xy,p));
    
    p*=dotSign;
    
    if(dotSign<0)
    {		
		if(IN.i==1.0)
		{
			IN.i=3.0;			
		}
		else if(IN.i==3.0)
		{
			IN.i=1.0;
		}
    }    
    
    float2 xyPos;
    //Check if the vertex is on a visible edge     
    if(IsContour(nA,nB,IN.v0))
    {		
		OUT.pos=TransformHalfQuadVertex(s0,s1,p,IN.i);
		OUT.color=edgeColor;
    }
    else if(IsRidgeValley(nA,nB,IN.v2,IN.v3))
    {
		OUT.pos=TransformThickVertex(s0,s1,p,IN.i);
		OUT.color=edgeColor;
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		OUT.pos=float4(0,0,-1,1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,0,0,1);
    }   	
	OUT.normal = Normal;	
	OUT.tex = float2(1,1);		

	return OUT;
}

VS_OUTPUT StartCapVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;       	 
    
    float4 Pos;    
    float3 Normal; 
    
    //Calculate s0 and s1 the screen space projections of the xy components of v0 and v1
    float4 v0 = float4(IN.v0,1.0);
    float4 v1 = float4(IN.v1,1.0);   
    float4 n0 = float4(IN.n0, 0.0);
    n0=normalize(n0);
    
    float4 s0 = mul(v0, worldViewProj);
    float4 s1 = mul(v1, worldViewProj);
    float4 m0 = mul((v0+n0), worldViewProj);
    
    s0.xy=(s0.xy/s0.w)*viewWidthHeight;
    s1.xy=(s1.xy/s1.w)*viewWidthHeight;
    m0.xy=((m0.xy/m0.w)*viewWidthHeight - s0.xy);         
    
    m0.xy=normalize(m0.xy)*edgeThickness;
    
    //Calculate p the screen space perpendicular to the edge
    float2 p = normalize(float2(s0.y-s1.y, s1.x-s0.x))*edgeThickness;  
    
    int dotSign = sign(dot(m0.xy,p));    
    p*=dotSign;     
    if(dotSign<0)
    {				
		IN.i=2-IN.i;
    }      
    
    float2 xyPos;
    //Check if the vertex is on a visible edge
    if(IsEdge(IN))// && !IsBackFace(IN))
    {		
		OUT.pos=TransformCapVertex(s0,m0,p,IN.i);
		OUT.color=edgeColor;		
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		OUT.pos=float4(0,0,-1,1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,0,0,1);
    }   	
	OUT.normal = Normal;		
	OUT.tex = float2(1,1);	

	return OUT;   
}

VS_OUTPUT EndCapVertex( VS_INPUT IN )
{
    VS_OUTPUT OUT;       	 
    
    float4 Pos;    
    float3 Normal; 
    
    //Calculate s0 and s1 the screen space projections of the xy components of v0 and v1
    float4 v0 = float4(IN.v0,1.0);
    float4 v1 = float4(IN.v1,1.0);   
    float4 n1 = float4(IN.n1, 0.0);
    n1=normalize(n1);
    
    float4 s0 = mul(v0, worldViewProj);
    float4 s1 = mul(v1, worldViewProj);
    float4 m1 = mul((v1+n1), worldViewProj);
    
    s0.xy=(s0.xy/s0.w)*viewWidthHeight;
    s1.xy=(s1.xy/s1.w)*viewWidthHeight;
    m1.xy=((m1.xy/m1.w)*viewWidthHeight - s1.xy);         
    
    m1.xy=normalize(m1.xy)*edgeThickness;
    
    //Calculate p the screen space perpendicular to the edge
    float2 p = normalize(float2(s0.y-s1.y, s1.x-s0.x))*edgeThickness;   
    
    int dotSign = sign(dot(m1.xy,p));    
    p*=dotSign;    
    if(dotSign<0)
    {				
		IN.i=2-IN.i;
    }     
    
    float2 xyPos;
    //Check if the vertex is on a visible edge
    if(IsEdge(IN))// && !IsBackFace(IN))
    {		
		OUT.pos=TransformCapVertex(s1,m1,p,IN.i);
		OUT.color=edgeColor;		
    }
    else
    {
		//The edge is not contour so set it behind the near plane so it won't be rendered
		OUT.pos=float4(0,0,-1,1);
		Normal=float3(0,0,1);
		OUT.color=float4(0,0,0,1);
    }   	
	OUT.normal = Normal;	
	OUT.tex = float2(1,1);		

	return OUT;  
}

float4 EdgePixel(VS_OUTPUT IN): COLOR
{	
	return IN.color * tex2D(EdgeSampler, IN.tex);	
}

technique TechniqueThinEdges
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 ThinEdgeVertex();
        PixelShader  = compile ps_2_0 EdgePixel();       
    }
}

technique TechniqueEdgeDetection
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 EdgeDetectionVertex();
        PixelShader  = compile ps_2_0 EdgePixel();       
    }
    
     pass Pass1
    {
		VertexShader = compile vs_2_0 StartCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
    
    pass Pass2
    {
		VertexShader = compile vs_2_0 EndCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
}

technique TechniqueThickEdges
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 ThickEdgeVertex();
        PixelShader  = compile ps_2_0 EdgePixel();       
    }    
    pass Pass1
    {
		VertexShader = compile vs_2_0 StartCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
    
    pass Pass2
    {
		VertexShader = compile vs_2_0 EndCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
}

technique TechniqueHalfQuadEdges
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 HalfQuadEdgeVertex();
        PixelShader  = compile ps_2_0 EdgePixel(); 
        AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;      
    }
    
    pass Pass1
    {
		VertexShader = compile vs_2_0 StartCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
    
    pass Pass2
    {
		VertexShader = compile vs_2_0 EndCapVertex();
		PixelShader  = compile ps_2_0 EdgePixel();   
    }
}