/******************************************************************************/
/*!
\file  sprinkler.fx
\author Jonathan Hurtado
\date   2/3/2010
\brief

  This shader is used to animate and color a particle generated from a particle
  system based on its lifetime

All content  2010 DigiPen (USA) Corporation, all rights reserved.
*/
/******************************************************************************/

float4x4 worldViewProj : WorldViewProjection;
float4x4 world : World;

uniform extern float4x4 gWVP;
uniform extern texture  gTex;
uniform extern float3   gEyePosW;
uniform extern float3   gAccel;
uniform extern float    gTime;
uniform extern int      gViewportHeight;
uniform extern float	gRotation;
uniform extern float	gFriction;
uniform extern float4	gColor;
uniform extern float4	gFinalColor;

sampler TexS = sampler_state
{
	Texture = <gTex>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = POINT;
	AddressU  = CLAMP;
    AddressV  = CLAMP;
};

struct INPUT_VS_PARTICLE
{
	float3 posL    : POSITION0; 
	float3 vel     : TEXCOORD0;
	float size     : TEXCOORD1;
	float fsize	   : TEXCOORD2;
	float time     : TEXCOORD3;
	float lifeTime : TEXCOORD4;
	float mass     : TEXCOORD5;

};

struct INPUT_VS_PARTICLE_INSTANCE
{
	float3 posL    : POSITION0; 
	float3 vel     : TEXCOORD0;
	float size     : TEXCOORD1;
	float fsize	   : TEXCOORD2;
	float time     : TEXCOORD3;
	float lifeTime : TEXCOORD4;
	float mass     : TEXCOORD5;

	float4 column1 : POSITION1;
	float4 column2 : POSITION2;
	float4 column3 : POSITION3;

};
 
struct VS_OUTPUT
{
    float4 posH  : POSITION0;
    float2 tex0  : TEXCOORD0; // D3D fills in for point sprites.
    float4 color : COLOR0;
	float4 rotation : COLOR1;
    float size   : PSIZE; // In pixels.
};

void TransformVertices(inout float3 Pos)
{	
    float3x3 rotation;
    rotation[0][0] = world[0][0];
    rotation[0][1] = world[0][1];
    rotation[0][2] = world[0][2];
    
    rotation[1][0] = world[1][0];
    rotation[1][1] = world[1][1];
    rotation[1][2] = world[1][2];
    
    rotation[2][0] = world[2][0];
    rotation[2][1] = world[2][1];
    rotation[2][2] = world[2][2];
    
	Pos = mul(Pos, rotation); // changes here
	Pos.x += world[3][0];
	Pos.y += world[3][1];
	Pos.z += world[3][2];
}

void TransformInstancedVertices(inout float3 Pos, in float4x4 world2)
{	
    float3x3 rotation;
    rotation[0][0] = world2[0][0];
    rotation[0][1] = world2[0][1];
    rotation[0][2] = world2[0][2];
    
    rotation[1][0] = world2[1][0];
    rotation[1][1] = world2[1][1];
    rotation[1][2] = world2[1][2];
    
    rotation[2][0] = world2[2][0];
    rotation[2][1] = world2[2][1];
    rotation[2][2] = world2[2][2];
    
	Pos = mul(Pos, rotation); // changes here
	Pos.x += world2[3][0];
	Pos.y += world2[3][1];
	Pos.z += world2[3][2];
}

void ComputeParticleRotation(in float rotation, out float4 rotationMatrix)
{
	if (rotation != 0)
	{
		//Compute a 2x2 rotation matrix
		float c = cos(rotation);
		float s = sin(rotation);

		rotationMatrix = float4(c, -s, s, c);

		// Normally we would output this matrix using a texture coordinate interpolator,
		// but texture coordinates are generated directly by the hardware when drawing
		// point sprites. So we have to use a color interpolator instead. Only trouble
		// is, color interpolators are clamped to the range 0 to 1. Our rotation values
		// range from -1 to 1, so we have to scale them to avoid unwanted clamping.
		
		rotationMatrix *= 0.5;
		rotationMatrix += 0.5;
 	}   

}

VS_OUTPUT SprinklerVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

	float3 Pos = IN.posL;
	
	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Constant acceleration.
	//Pos = Pos + IN.vel*t*(1.0f-gFriction) + 0.5f * gAccel * t * t;	
	Pos = Pos + IN.vel*t + 0.5f * gAccel * t * t;

	//TransformVertices(Pos);

	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(IN.posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
	//outVS.posH = mul(float4(IN.posL, 1.0f), worldViewProj);
		
	// Compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	float d = distance(Pos, gEyePosW);
	
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);
	
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	//outVS.color.a = 1.0f;
	//outVS.color = (IN.fcolor*(t/IN.lifeTime)) + (IN.color*((IN.lifeTime-t)/IN.lifeTime));
	//outVS.color = IN.fcolor;
	// Done--return the output.

	float rotation = gRotation * t;
	float4 rotationMatrix = 0;
	ComputeParticleRotation(rotation, rotationMatrix);
	outVS.rotation = rotationMatrix;

    return outVS;
}

VS_OUTPUT SprinklerLocalToWorldVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

	float3 Pos = IN.posL;
	
	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Constant acceleration.
	//Pos = Pos + IN.vel*t*(1.0f-gFriction) + 0.5f * gAccel * t * t;	
	Pos = Pos + IN.vel*t + 0.5f * gAccel * t * t;

	TransformVertices(Pos);

	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(IN.posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
	//outVS.posH = mul(float4(IN.posL, 1.0f), worldViewProj);
		
	// Compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	float d = distance(Pos, gEyePosW);
	
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);
	
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	//outVS.color.a = 1.0f;
	//outVS.color = (IN.fcolor*(t/IN.lifeTime)) + (IN.color*((IN.lifeTime-t)/IN.lifeTime));
	//outVS.color = IN.fcolor;
	// Done--return the output.

	float rotation = gRotation * t;
	float4 rotationMatrix = 0;
	ComputeParticleRotation(rotation, rotationMatrix);
	outVS.rotation = rotationMatrix;

    return outVS;
}

VS_OUTPUT Instance_SprinklerVS(INPUT_VS_PARTICLE_INSTANCE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

	float4x4 world2 =
	{
		IN.column1.x, IN.column2.x, IN.column3.x, 0.0f,
		IN.column1.y, IN.column2.y, IN.column3.y, 0.0f,
		IN.column1.z, IN.column2.z, IN.column3.z, 0.0f,
		IN.column1.w, IN.column2.w, IN.column3.w, 1.0f,
	};

	float3 Pos = IN.posL;
	
	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Constant acceleration.
	//Pos = Pos + IN.vel*t*(1.0f-gFriction) + 0.5f * gAccel * t * t;	
	Pos = Pos + IN.vel*t + 0.5f * gAccel * t * t;

	TransformInstancedVertices(Pos, world2);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(IN.posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
	//outVS.posH = mul(float4(IN.posL, 1.0f), worldViewProj);
		
	// Compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	float d = distance(Pos, gEyePosW);
	
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);

	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	
	//outVS.color = (IN.fcolor*(t/IN.lifeTime)) + (IN.color*((IN.lifeTime-t)/IN.lifeTime));
	//outVS.color = IN.fcolor;
	// Done--return the output.

	float rotation = gRotation * t;
	float4 rotationMatrix = 0;
	ComputeParticleRotation(rotation, rotationMatrix);
	outVS.rotation = rotationMatrix;

    return outVS;
}


VS_OUTPUT SpiralVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

 	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Rotate the particles about local space.
	float thetaSine, thetaCosine;
	
	//sincos(t * IN.mass, thetaSine, thetaCosine);
	//IN.posL.x = IN.posL.x + IN.mass * thetaCosine;
	//IN.posL.y = IN.posL.y + IN.mass * thetaSine;

	sincos(t * 10, thetaSine, thetaCosine);
	IN.posL.x = IN.posL.x + t * IN.mass * thetaCosine;
	IN.posL.y = IN.posL.y + t * IN.mass * thetaSine;

	//float sine, cosine;
	//sincos(0.5f * IN.mass * t, sine, cosine);
	//float x = IN.posL.x * cosine + IN.posL.y * -sine;
	//float y = IN.posL.x * sine + IN.posL.y * cosine;
	
	// Oscillate particles up and down.
	//float s = sin(6.0f*t);
	//IN.posL.x = x;
	//IN.posL.y = y + IN.mass * s;
	
	// Constant acceleration.
	//IN.posL = IN.posL + IN.vel*t + 0.5f * gAccel * t * t;

	float3 Pos = IN.posL;

	//TransformVertices(Pos);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
		
	// Ramp up size over time.
	//IN.size += 1.0f*t*t;
	
	// Also compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	//float d = distance(Pos, gEyePosW);
	//outVS.size = gViewportHeight * IN.size/(1.0f + 8.0f*d);

	float d = distance(Pos, gEyePosW);
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);

	// Fade color from white to black over time to fade particles out.
	//outVS.color = (1.0f - (t / IN.lifeTime));
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	
	// Done--return the output.
    return outVS;
}

VS_OUTPUT SpiralLocaltoWorldVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

 	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Rotate the particles about local space.
	float thetaSine, thetaCosine;
	
	//sincos(t * IN.mass, thetaSine, thetaCosine);
	//IN.posL.x = IN.posL.x + IN.mass * thetaCosine;
	//IN.posL.y = IN.posL.y + IN.mass * thetaSine;

	sincos(t * 10, thetaSine, thetaCosine);
	IN.posL.x = IN.posL.x + t * IN.mass * thetaCosine;
	IN.posL.y = IN.posL.y + t * IN.mass * thetaSine;

	//float sine, cosine;
	//sincos(0.5f * IN.mass * t, sine, cosine);
	//float x = IN.posL.x * cosine + IN.posL.y * -sine;
	//float y = IN.posL.x * sine + IN.posL.y * cosine;
	
	// Oscillate particles up and down.
	//float s = sin(6.0f*t);
	//IN.posL.x = x;
	//IN.posL.y = y + IN.mass * s;
	
	// Constant acceleration.
	//IN.posL = IN.posL + IN.vel*t + 0.5f * gAccel * t * t;

	float3 Pos = IN.posL;

	TransformVertices(Pos);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
		
	// Ramp up size over time.
	//IN.size += 1.0f*t*t;
	
	// Also compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	//float d = distance(Pos, gEyePosW);
	//outVS.size = gViewportHeight * IN.size/(1.0f + 8.0f*d);

	float d = distance(Pos, gEyePosW);
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);

	// Fade color from white to black over time to fade particles out.
	//outVS.color = (1.0f - (t / IN.lifeTime));
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	
	// Done--return the output.
    return outVS;
}

VS_OUTPUT ReverseSpiralVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

 	// Get age of particle from creation time.
	float t = IN.lifeTime - (gTime - IN.time);
	
	// Rotate the particles about local space.
	float thetaSine, thetaCosine;
	
	//sincos(t * IN.mass, thetaSine, thetaCosine);
	//IN.posL.x = IN.posL.x + IN.mass * thetaCosine;
	//IN.posL.y = IN.posL.y + IN.mass * thetaSine;

	sincos(t * 10, thetaSine, thetaCosine);
	IN.posL.x = IN.posL.x + t * IN.mass * thetaCosine;
	IN.posL.y = IN.posL.y + t * IN.mass * thetaSine;

	//float sine, cosine;
	//sincos(0.5f * IN.mass * t, sine, cosine);
	//float x = IN.posL.x * cosine + IN.posL.y * -sine;
	//float y = IN.posL.x * sine + IN.posL.y * cosine;
	
	// Oscillate particles up and down.
	//float s = sin(6.0f*t);
	//IN.posL.x = x;
	//IN.posL.y = y + IN.mass * s;
	
	// Constant acceleration.
	//IN.posL = IN.posL + IN.vel*t + 0.5f * gAccel * t * t;

	float3 Pos = IN.posL;

	//TransformVertices(Pos);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
		
	// Ramp up size over time.
	//IN.size += 1.0f*t*t;
	
	// Also compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	//float d = distance(Pos, gEyePosW);
	//outVS.size = gViewportHeight * IN.size/(1.0f + 8.0f*d);

	float d = distance(Pos, gEyePosW);
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);

	// Fade color from white to black over time to fade particles out.
	//outVS.color = (1.0f - (t / IN.lifeTime));
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	
	// Done--return the output.
    return outVS;
}

VS_OUTPUT ReverseSpiralLocaltoWorldVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

 	// Get age of particle from creation time.
	float t = IN.lifeTime - (gTime - IN.time);
	
	// Rotate the particles about local space.
	float thetaSine, thetaCosine;
	
	//sincos(t * IN.mass, thetaSine, thetaCosine);
	//IN.posL.x = IN.posL.x + IN.mass * thetaCosine;
	//IN.posL.y = IN.posL.y + IN.mass * thetaSine;

	sincos(t * 10, thetaSine, thetaCosine);
	IN.posL.x = IN.posL.x + t * IN.mass * thetaCosine;
	IN.posL.y = IN.posL.y + t * IN.mass * thetaSine;

	//float sine, cosine;
	//sincos(0.5f * IN.mass * t, sine, cosine);
	//float x = IN.posL.x * cosine + IN.posL.y * -sine;
	//float y = IN.posL.x * sine + IN.posL.y * cosine;
	
	// Oscillate particles up and down.
	//float s = sin(6.0f*t);
	//IN.posL.x = x;
	//IN.posL.y = y + IN.mass * s;
	
	// Constant acceleration.
	//IN.posL = IN.posL + IN.vel*t + 0.5f * gAccel * t * t;

	float3 Pos = IN.posL;

	TransformVertices(Pos);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
		
	// Ramp up size over time.
	//IN.size += 1.0f*t*t;
	
	// Also compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	//float d = distance(Pos, gEyePosW);
	//outVS.size = gViewportHeight * IN.size/(1.0f + 8.0f*d);

	float d = distance(Pos, gEyePosW);
	float finalScale = IN.size*IN.fsize;
	float scaleFactor = (finalScale*(t/IN.lifeTime)) + (IN.size*((IN.lifeTime-t)/IN.lifeTime));	
	outVS.size = gViewportHeight * scaleFactor/(1.0f + 8.0f*d);

	// Fade color from white to black over time to fade particles out.
	//outVS.color = (1.0f - (t / IN.lifeTime));
	outVS.color = (gFinalColor * (t/IN.lifeTime)) + (gColor * ((IN.lifeTime-t)/IN.lifeTime));
	
	// Done--return the output.
    return outVS;
}

VS_OUTPUT LingerVS(INPUT_VS_PARTICLE IN)
{
    // Zero out our output.
	VS_OUTPUT outVS = (VS_OUTPUT)0;

 	// Get age of particle from creation time.
	float t = gTime - IN.time;
	
	// Rotate the particles about local space.
	float sine, cosine;
	sincos(0.5f * IN.mass * t, sine, cosine);
	float x = IN.posL.x * cosine + IN.posL.y * -sine;
	float y = IN.posL.x * sine + IN.posL.y * cosine;
	
	// Oscillate particles up and down.
	float s = sin(6.0f*t);
	IN.posL.x = x;
	IN.posL.y = y + IN.mass * s;
	
	// Constant acceleration.
	IN.posL = IN.posL + IN.vel*t + 0.5f * gAccel * t * t;

	float3 Pos = IN.posL;

	TransformVertices(Pos);
	
	// Transform to homogeneous clip space.
	//outVS.posH = mul(float4(posL, 1.0f), gWVP);
	outVS.posH = mul( float4(Pos, 1) , worldViewProj);	
		
	// Ramp up size over time.
	IN.size += 1.0f*t*t;
	
	// Also compute size as a function of the distance from the camera,
	// and the viewport heights.  The constants were found by 
	// experimenting.
	//float d = distance(IN.posL, gEyePosL);
	float d = distance(Pos, gEyePosW);
	outVS.size = gViewportHeight * IN.size/(1.0f + 8.0f*d);
	
	// Fade color from white to black over time to fade particles out.
	outVS.color = (1.0f - (t / IN.lifeTime));
	
	// Done--return the output.
    return outVS;
}

float4 SprinklerPS(VS_OUTPUT IN) : COLOR
{
	return tex2D(TexS, IN.tex0) * IN.color;
}

float4 Sprinkler_RotationPS(VS_OUTPUT IN) : COLOR
{
	float2 texCoord = IN.tex0;

    // We want to rotate around the middle of the particle, not the origin,
    // so we offset the texture coordinate accordingly.
    texCoord -= 0.5;
    
    // Apply the rotation matrix, after rescaling it back from the packed
    // color interpolator format into a full -1 to 1 range.
    float4 rotation = IN.rotation * 2 - 1;
    
    texCoord = mul(texCoord, float2x2(rotation));
    
    // Point sprites are squares. So are textures. When we rotate one square
    // inside another square, the corners of the texture will go past the
    // edge of the point sprite and get clipped. To avoid this, we scale
    // our texture coordinates to make sure the entire square can be rotated
    // inside the point sprite without any clipping.
    texCoord *= sqrt(2) * 0.6;
    
    // Undo the offset used to control the rotation origin.
    texCoord += 0.5;

    return tex2D(TexS, texCoord) * IN.color;

}


technique SprinklerTech
{
    pass P0
    {
        vertexShader = compile vs_3_0 SprinklerVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique SprinklerTechLocaltoWorld
{
    pass P0
    {
        vertexShader = compile vs_3_0 SprinklerLocalToWorldVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique SprinklerTechRotation
{
    pass P0
    {
        vertexShader = compile vs_3_0 SprinklerVS();
        pixelShader  = compile ps_3_0 Sprinkler_RotationPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;

	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique SprinklerTechRotationLocaltoWorld
{
    pass P0
    {
        vertexShader = compile vs_3_0 SprinklerLocalToWorldVS();
        pixelShader  = compile ps_3_0 Sprinkler_RotationPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;

	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique InstanceSprinklerTech
{
    pass P0
    {
        vertexShader = compile vs_3_0 Instance_SprinklerVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = One;
	    DestBlend    = One;
	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique InstanceSprinklerTechRotation
{
    pass P0
    {
        vertexShader = compile vs_3_0 Instance_SprinklerVS();
        pixelShader  = compile ps_3_0 Sprinkler_RotationPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = One;
	    DestBlend    = One;

	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique SpiralTech
{
    pass P0
    {
        vertexShader = compile vs_3_0 SpiralVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique ReverseSpiralTech
{
    pass P0
    {
        vertexShader = compile vs_3_0 ReverseSpiralVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique SpiralTechLocaltoWorld
{
    pass P0
    {
        vertexShader = compile vs_3_0 SpiralLocaltoWorldVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}

technique ReverseSpiralTechLocaltoWorld
{
    pass P0
    {
        vertexShader = compile vs_3_0 ReverseSpiralLocaltoWorldVS();
        pixelShader  = compile ps_3_0 SprinklerPS();
        
        PointSpriteEnable = true;
        AlphaBlendEnable = true;
	    SrcBlend     = SrcAlpha;
	    DestBlend    = InvSrcAlpha;
	    	    
	    // Don't write to depth buffer; that is, particles don't obscure
	    // the objects behind them.
	    ZWriteEnable = false;
    }
}